@@ -44,6 +44,7 @@ int	s_trail, ntrail, verbose, xspin, notabs, rvopt;
 int	no_print, no_wrapup, Caccess, limited_vis, like_java;
 int	separate;	/* separate compilation */
 int	export_ast;	/* pangen5.c */
+int	norecompile;	/* main.c */
 int	old_scope_rules;	/* use pre 5.3.0 rules */
 int	old_priority_rules;	/* use pre 6.2.0 rules */
 int	product, Strict;
@@ -52,7 +53,7 @@ short	replay;
 int	merger = 1, deadvar = 1, implied_semis = 1;
 int	ccache = 0; /* oyvind teig: 5.2.0 case caching off by default */
 
-static int preprocessonly, SeedUsed, itsr, itsr_n;
+static int preprocessonly, SeedUsed, itsr, itsr_n, sw_or_bt;
 static int seedy;	/* be verbose about chosen seed */
 static int inlineonly;	/* show inlined code */
 static int dataflow = 1;
@@ -159,7 +160,7 @@ final_fiddle(void)
 }
 
 static int
-change_param(char *t, char *what, int range)
+change_param(char *t, char *what, int range, int bottom)
 {	char *ptr;
 	int v;
 
@@ -169,7 +170,10 @@ change_param(char *t, char *what, int range)
 		if (!isdigit((int) *ptr))
 		{	return 0;
 		}
-		v = (atoi(ptr) + 1)%range;
+		v = atoi(ptr) + 1;	/* was: v = (atoi(ptr)+1)%range */
+		if (v >= range)
+		{	v = bottom;
+		}
 		if (v >= 100)
 		{	*ptr++ = '0' + (v/100); v %= 100;
 			*ptr++ = '0' + (v/10);
@@ -237,6 +241,7 @@ string_trim(char *t)
 int
 e_system(int v, const char *s)
 {	static int count = 1;
+
 	/* v == 0 : checks to find non-linked version of gcc */
 	/* v == 1 : all other commands */
 	/* v == 2 : preprocessing the promela input */
@@ -460,25 +465,33 @@ recompile:
 		if (has_ltl)
 		{	(void) unlink("_spin_nvr.tmp");
 		}
+
+		if (!norecompile)
+		{	/* make sure that if compilation fails we do not continue */
 #ifdef PC
-		/* make sure that if compilation fails we do not continue */
-		(void) unlink("./pan.exe");
+			(void) unlink("./pan.exe");
 #else
-		(void) unlink("./pan");
+			(void) unlink("./pan");
 #endif
+		}
 runit:
-		estatus = e_system(1, tmp);	/* compile */
+		if (norecompile && tmp != tmp2)
+		{	estatus = 0;
+		} else
+		{	estatus = e_system(1, tmp);	/* compile or run */
+		}
 		if (replay || estatus < 0)
 		{	goto skipahead;
 		}
 		/* !replay */
-		if (itsr == 0)			/* single run */
+		if (itsr == 0 && !sw_or_bt)			/* single run */
 		{	estatus = e_system(1, tmp2);
 		} else if (itsr > 0)	/* iterative search refinement */
 		{	int is_swarm = 0;
 			if (tmp2 != tmp)	/* swarm: did only compilation so far */
 			{	tmp = tmp2;	/* now we point to the run command */
 				estatus = e_system(1, tmp);	/* first run */
+				itsr--;
 			}
 			itsr--;			/* count down */
 
@@ -493,12 +506,12 @@ runit:
 
 			/* increase -w every itsr_n-th run */
 			if ((itsr_n > 0 && (itsr == 0 || (itsr%itsr_n) != 0))
-			||  (change_param(tmp, "-w", 36) >= 0))	/* max 4G bit statespace */
-			{	(void) change_param(tmp, "-h", 500);	/* hash function 0.499 */
-				(void) change_param(tmp, "-p_rotate", 256); /* if defined */
-				(void) change_param(tmp, "-k", 4);	/* nr bits per state 0->1,1,2,3 */
-				(void) change_param(tmp, "-T", 2);	/* with or without t_reverse*/
-				(void) change_param(tmp, "-P", 2);	/* -P 0..1 != p_reverse */
+			||  (change_param(tmp, "-w", 36, 18) >= 0))	/* max 4G bit statespace */
+			{	(void) change_param(tmp, "-h", 500, 0);	/* hash function 0.499 */
+				(void) change_param(tmp, "-p_rotate", 256, 0); /* if defined */
+				(void) change_param(tmp, "-k", 4, 1);	/* nr bits per state 0->1,2,3 */
+				(void) change_param(tmp, "-T", 2, 0);	/* with or without t_reverse*/
+				(void) change_param(tmp, "-P", 2, 0);	/* -P 0..1 != p_reverse */
 				change_rs(tmp);			/* change random seed */
 				string_trim(tmp);
 				if (rand()%5 == 0)		/* 20% of all runs */
@@ -568,9 +581,9 @@ preprocess(char *a, char *b, int a_tmp)
 	   the code below assumes we are on a cygwin-like system
 	 */
 	if (strncmp(PreProc, "gcc ", strlen("gcc ")) == 0)
-	{	if (e_system(0, "gcc-4 --version > pan.pre") == 0)
+	{	if (e_system(0, "gcc-4 --version > pan.pre 2>&1") == 0)
 		{	strcpy(PreProc, "gcc-4 -std=gnu99 -E -x c");
-		} else if (e_system(0, "gcc-3 --version > pan.pre") == 0)
+		} else if (e_system(0, "gcc-3 --version > pan.pre 2>&1") == 0)
 		{	strcpy(PreProc, "gcc-3 -std=gnu99 -E -x c");
 	}	}
 #endif
@@ -644,25 +657,31 @@ usage(void)
 	printf("\t	if the model contains embedded c-code, the ./pan executable is used\n");
 	printf("\t	otherwise spin itself is used to replay the trailfile\n");
 	printf("\t	note that pan recognizes different runtime options than spin itself\n");
-	printf("\t-search  (or -run) generate a verifier, and compile and run it\n");
+	printf("\t-run  (or -search) generate a verifier, and compile and run it\n");
 	printf("\t      options before -search are interpreted by spin to parse the input\n");
 	printf("\t      options following a -search are used to compile and run the verifier pan\n");
 	printf("\t	    valid options that can follow a -search argument include:\n");
 	printf("\t	    -bfs	perform a breadth-first search\n");
 	printf("\t	    -bfspar	perform a parallel breadth-first search\n");
+	printf("\t	    -dfspar	perform a parallel depth-first search, same as -DNCORE=4\n");
 	printf("\t	    -bcs	use the bounded-context-switching algorithm\n");
 	printf("\t	    -bitstate	or -bit, use bitstate storage\n");
-	printf("\t	    -biterate	use bitstate with iterative search refinement (-w18..-w35)\n");
-	printf("\t	    -swarmN,M like -biterate, but running all iterations in parallel\n");
-	printf("\t			perform N parallel runs and increment -w every M runs\n");
+	printf("\t	    -biterateN,M use bitstate with iterative search refinement (-w18..-w35)\n");
+	printf("\t			perform N randomized runs and increment -w every M runs\n");
 	printf("\t			default value for N is 10, default for M is 1\n");
+	printf("\t			(use N,N to keep -w fixed for all runs)\n");
+	printf("\t			(add -w to see which commands will be executed)\n");
+	printf("\t			(add -W if ./pan exists and need not be recompiled)\n");
+	printf("\t	    -swarmN,M like -biterate, but running all iterations in parallel\n");
 	printf("\t	    -link file.c  link executable pan to file.c\n");
 	printf("\t	    -collapse	use collapse state compression\n");
+	printf("\t	    -noreduce	do not use partial order reduction\n");
 	printf("\t	    -hc  	use hash-compact storage\n");
 	printf("\t	    -noclaim	ignore all ltl and never claims\n");
 	printf("\t	    -p_permute	use process scheduling order random permutation\n");
 	printf("\t	    -p_rotateN	use process scheduling order rotation by N\n");
 	printf("\t	    -p_reverse	use process scheduling order reversal\n");
+	printf("\t	    -rhash      randomly pick one of the -p_... options\n");
 	printf("\t	    -ltl p	verify the ltl property named p\n");
 	printf("\t	    -safety	compile for safety properties only\n");
 	printf("\t	    -i	    	use the dfs iterative shortening algorithm\n");
@@ -765,10 +784,13 @@ static struct {
 	{ "-hc",	"-DHC4",	0 },
 	{ "-collapse",	"-DCOLLAPSE",	0 },
 	{ "-noclaim",	"-DNOCLAIM",	0 },
+	{ "-noreduce",	"-DNOREDUCE",	0 },
+	{ "-np",	"-DNP",		0 },
 	{ "-permuted",	"-DPERMUTED",	0 },
 	{ "-p_permute", "-DPERMUTED",	1 },
 	{ "-p_rotate",	"-DPERMUTED",	1 },
 	{ "-p_reverse",	"-DPERMUTED",	1 },
+	{ "-rhash",	"-DPERMUTED",	1 },
 	{ "-safety",	"-DSAFETY",	0 },
 	{ "-i",		"-DREACH",	1 },
 	{ "-l",		"-DNP",		1 },
@@ -795,6 +817,7 @@ add_runtime(char *s)
 
 	if (strncmp(s, "-biterate", strlen("-biterate")) == 0)
 	{	itsr = 10;	/* default nr of sequential iterations */
+		sw_or_bt = 1;
 		if (isdigit((int) s[9]))
 		{	itsr = atoi(&s[9]);
 			if (itsr < 1)
@@ -806,6 +829,7 @@ add_runtime(char *s)
 	}
 	if (strncmp(s, "-swarm", strlen("-swarm")) == 0)
 	{	itsr = -10;	/* parallel iterations */
+		sw_or_bt = 1;
 		if (isdigit((int) s[6]))
 		{	itsr = atoi(&s[6]);
 			if (itsr < 1)
@@ -825,6 +849,10 @@ add_runtime(char *s)
 			}
 			return;
 	}	}
+	if (strncmp(s, "-dfspar", strlen("-dfspar")) == 0)
+	{	add_comptime("-DNCORE=4");
+		return;
+	}
 
 	tmp = (char *) emalloc(strlen(pan_runtime)+strlen(s)+2);
 	sprintf(tmp, "%s %s", pan_runtime, s);
@@ -903,7 +931,7 @@ samecase:			if (buzzed != 0)
 				buzzed  = 2;
 				analyze = 1;
 				argc--; argv++;
-				/* process all remaining arguments as relating to pan: */
+				/* process all remaining arguments, except -w/-W, as relating to pan: */
 				while (argc > 1 && argv[1][0] == '-')
 				{ switch (argv[1][1]) {
 				  case 'D': /* eg -DNP */
@@ -912,6 +940,14 @@ samecase:			if (buzzed != 0)
 				  case 'U': /* to undefine a macro */
 					add_comptime(argv[1]);
 					break;
+#if 0
+				  case 'w': /* conflicts with bitstate runtime arg */
+					verbose += 64; 
+					break;
+#endif
+				  case 'W':
+					norecompile = 1;
+					break;
 				  case 'l':
 					if (strcmp(&argv[1][1], "ltl") == 0)
 					{	add_runtime("-N");
@@ -965,6 +1001,7 @@ samecase:			if (buzzed != 0)
 			  alldone(0);
 			  break;
 		case 'w': verbose += 64; break;
+		case 'W': norecompile = 1; break;	/* 6.4.7: for swarm/biterate */
 		case 'x': /* internal - reserved use */
 			  if (buzzed != 0)
 			  {	fatal("cannot combine -x with -run -search or -replay", (char *)0);
@@ -1152,7 +1189,6 @@ ltl_list(char *nm, char *fm)
 			}
 			tl_out = fd_ltl;
 		}
-
 		add_ltl = (char **) emalloc(5 * sizeof(char *));
 		add_ltl[1] = "-c";
 		add_ltl[2] = nm;
@@ -1382,6 +1418,7 @@ rem_var(Symbol *a, Lextok *b, Symbol *c, Lextok *ndx)
 	has_remote++;
 	has_remvar++;
 	dataflow = 0;	/* turn off dead variable resets 4.2.5 */
+	merger   = 0;	/* turn off statement merging for locals 6.4.9 */
 	tmp1 = nn(ZN, '?', b, ZN); tmp1->sym = a;
 	tmp1 = nn(ZN, 'p', tmp1, ndx);
 	tmp1->sym = c;
