@@ -16,7 +16,6 @@
 
 extern RunList	*X;
 extern Symbol	*Fname;
-extern Lextok	*Mtype;
 extern int	verbose, TstOnly, s_trail, analyze, columns;
 extern int	lineno, depth, xspin, m_loss, jumpsteps;
 extern int	nproc, nstop;
@@ -35,8 +34,10 @@ static int	a_rcv(Queue *, Lextok *, int);
 static int	a_snd(Queue *, Lextok *);
 static int	sa_snd(Queue *, Lextok *);
 static int	s_snd(Queue *, Lextok *);
-extern void	sr_buf(int, int);
-extern void	sr_mesg(FILE *, int, int);
+extern Lextok	**find_mtype_list(const char *);
+extern char	*which_mtype(const char *);
+extern void	sr_buf(int, int, const char *);
+extern void	sr_mesg(FILE *, int, int, const char *);
 extern void	putarrow(int, int);
 static void	sr_talk(Lextok *, int, char *, char *, int, Queue *);
 
@@ -81,14 +82,18 @@ qmake(Symbol *s)
 
 	q->contents  = (int *) emalloc(q->nflds*i*sizeof(int));
 	q->fld_width = (int *) emalloc(q->nflds*sizeof(int));
-	q->stepnr = (int *)   emalloc(i*sizeof(int));
+	q->mtp       = (char **) emalloc(q->nflds*sizeof(char *));
+	q->stepnr    = (int *) emalloc(i*sizeof(int));
 
 	for (m = s->ini->rgt, i = 0; m; m = m->rgt)
 	{	if (m->sym && m->ntyp == STRUCT)
-			i = Width_set(q->fld_width, i, getuname(m->sym));
-		else
+		{	i = Width_set(q->fld_width, i, getuname(m->sym));
+		} else
+		{	if (m->sym)
+			{	q->mtp[i] = m->sym->name;
+			}
 			q->fld_width[i++] = m->ntyp;
-	}
+	}	}
 	q->nxt = qtab;
 	qtab = q;
 	ltab[q->qid-1] = q;
@@ -261,6 +266,40 @@ typ_ck(int ft, int at, char *s)
 	}
 }
 
+static void
+mtype_ck(char *p, Lextok *arg)
+{	char *t, *s = p?p:"_unnamed_";
+
+	if (!arg
+	||  !arg->sym)
+	{	return;
+	}
+
+	switch (arg->ntyp) {
+	case NAME:
+		if (arg->sym->mtype_name)
+		{	t = arg->sym->mtype_name->name;
+		} else
+		{	t = "_unnamed_";
+		}
+		break;
+	case CONST:
+		t = which_mtype(arg->sym->name);
+		break;
+	default:
+		t = "expression";
+		break;
+	}
+
+	if (strcmp(s, t) != 0)
+	{	printf("spin: %s:%d, Error: '%s' is type '%s', but ",
+			arg->fn?arg->fn->name:"", arg->ln,
+			arg->sym->name, t);
+		printf("should be type '%s'\n", s);
+		non_fatal("incorrect type of '%s'", arg->sym->name);
+	}
+}
+
 static int
 a_snd(Queue *q, Lextok *n)
 {	Lextok *m;
@@ -279,10 +318,17 @@ a_snd(Queue *q, Lextok *n)
 	for (m = n->rgt; m && j < q->nflds; m = m->rgt, j++)
 	{	int New = eval(m->lft);
 		q->contents[i+j] = cast_val(q->fld_width[j], New, 0);
+
+		if (q->fld_width[i+j] == MTYPE)
+		{	mtype_ck(q->mtp[i+j], m->lft);	/* 6.4.8 */
+		}
+
 		if ((verbose&16) && depth >= jumpsteps)
-			sr_talk(n, New, "Send ", "->", j, q);
-		typ_ck(q->fld_width[j], Sym_typ(m->lft), "send");
+			sr_talk(n, New, "Send ", "->", i+j, q);
+
+		typ_ck(q->fld_width[i+j], Sym_typ(m->lft), "send");
 	}
+
 	if ((verbose&16) && depth >= jumpsteps)
 	{	for (i = j; i < q->nflds; i++)
 			sr_talk(n, 0, "Send ", "->", i, q);
@@ -304,10 +350,16 @@ a_rcv(Queue *q, Lextok *n, int full)
 	extern int Rvous;
 
 	if (q->qlen == 0)
-		return 0;	/* q is empty */
+	{	return 0;	/* q is empty */
+	}
 try_slot:
 	/* test executability */
 	for (m = n->rgt, j=0; m && j < q->nflds; m = m->rgt, j++)
+	{
+		if (q->fld_width[i*q->nflds+j] == MTYPE)
+		{	mtype_ck(q->mtp[i*q->nflds+j], m->lft);	/* 6.4.8 */
+		}
+
 		if ((m->lft->ntyp == CONST
 		   && q->contents[i*q->nflds+j] != m->lft->val)
 		||  (m->lft->ntyp == EVAL
@@ -317,17 +369,18 @@ try_slot:
 			|| ++i >= q->qlen)	/* last slot */
 				return 0;	/* no match  */
 			goto try_slot;
-		}
+	}	}
+
 	if (TstOnly) return 1;
 
 	if (verbose&8)
 	{	if (j < q->nflds)
-			printf("%3d: warning: missing params in next recv\n",
+		{	printf("%3d: warning: missing params in next recv\n",
 				depth);
-		else if (m)
-			printf("%3d: warning: too many params in next recv\n",
+		} else if (m)
+		{	printf("%3d: warning: too many params in next recv\n",
 				depth);
-	}
+	}	}
 
 	/* set the fields */
 	if (Rvous)
@@ -381,7 +434,11 @@ s_snd(Queue *q, Lextok *n)
 	for (m = n->rgt; m && j < q->nflds; m = m->rgt, j++)
 	{	q->contents[j] = cast_val(q->fld_width[j], eval(m->lft), 0);
 		typ_ck(q->fld_width[j], Sym_typ(m->lft), "rv-send");
-	}
+
+		if (q->fld_width[j] == MTYPE)
+		{	mtype_ck(q->mtp[j], m->lft);	/* 6.4.8 */
+	}	}
+
 	q->qlen = 1;
 	if (!complete_rendez())
 	{	q->qlen = 0;
@@ -395,27 +452,40 @@ s_snd(Queue *q, Lextok *n)
 	if ((verbose&16) && depth >= jumpsteps)
 	{	m = n->rgt;
 		rX = X; X = sX;
+
 		for (j = 0; m && j < q->nflds; m = m->rgt, j++)
-			sr_talk(n, eval(m->lft), "Sent ", "->", j, q);
+		{	sr_talk(n, eval(m->lft), "Sent ", "->", j, q);
+		}
+
 		for (i = j; i < q->nflds; i++)
-			sr_talk(n, 0, "Sent ", "->", i, q);
+		{	sr_talk(n, 0, "Sent ", "->", i, q);
+		}
+
 		if (j < q->nflds)
-			  printf("%3d: warning: missing params in rv-send\n",
+		{	  printf("%3d: warning: missing params in rv-send\n",
 				depth);
-		else if (m)
-			  printf("%3d: warning: too many params in rv-send\n",
+		} else if (m)
+		{	  printf("%3d: warning: too many params in rv-send\n",
 				depth);
+		}
+
 		X = rX;	/* restore receiver's context */
 		if (!s_trail)
 		{	if (!n_rem || !q_rem)
 				fatal("cannot happen, s_snd", (char *) 0);
 			m = n_rem->rgt;
 			for (j = 0; m && j < q->nflds; m = m->rgt, j++)
-			{	if (m->lft->ntyp != NAME
-				||  strcmp(m->lft->sym->name, "_") != 0)
-					i = eval(m->lft);
-				else	i = 0;
+			{
+				if (q->fld_width[j] == MTYPE)
+				{	mtype_ck(q->mtp[j], m->lft);	/* 6.4.8 */
+				}
 
+				if (m->lft->ntyp != NAME
+				||  strcmp(m->lft->sym->name, "_") != 0)
+				{	i = eval(m->lft);
+				} else
+				{	i = 0;
+				}
 				if (verbose&8)
 				sr_talk(n_rem,i,"Recv ","<-",j,q_rem);
 			}
@@ -471,7 +541,7 @@ difcolumns(Lextok *n, char *tr, int v, int j, Queue *q)
 	} else
 		strcat(Buf, ",");
 	if (tr[0] == '[') strcat(Buf, "[");
-	sr_buf(v, q->fld_width[j] == MTYPE);
+	sr_buf(v, q->fld_width[j] == MTYPE, q->mtp[j]);
 	if (j == q->nflds - 1)
 	{	int cnr;
 		if (s_trail)
@@ -507,7 +577,7 @@ docolumns(Lextok *n, char *tr, int v, int j, Queue *q)
 	} else
 		printf(",");
 	if (tr[0] == '[') printf("[");
-	sr_mesg(stdout, v, q->fld_width[j] == MTYPE);
+	sr_mesg(stdout, v, q->fld_width[j] == MTYPE, q->mtp[j]);
 	if (j == q->nflds - 1)
 	{	if (tr[0] == '[') printf("]");
 		printf("\n");
@@ -576,7 +646,7 @@ sr_talk(Lextok *n, int v, char *tr, char *a, int j, Queue *q)
 		}
 	} else
 		printf(",");
-	sr_mesg(stdout, v, q->fld_width[j] == MTYPE);
+	sr_mesg(stdout, v, q->fld_width[j] == MTYPE, q->mtp[j]);
 
 	if (j == q->nflds - 1)
 	{	if (xspin)
@@ -593,12 +663,16 @@ sr_talk(Lextok *n, int v, char *tr, char *a, int j, Queue *q)
 }
 
 void
-sr_buf(int v, int j)
+sr_buf(int v, int j, const char *s)
 {	int cnt = 1; Lextok *n;
 	char lbuf[512];
+	Lextok *Mtype = ZN;
 
+	if (j)
+	{	Mtype = *find_mtype_list(s?s:"_unnamed_");
+	}
 	for (n = Mtype; n && j; n = n->rgt, cnt++)
-		if (cnt == v)
+	{	if (cnt == v)
 		{	if(strlen(n->lft->sym->name) >= sizeof(lbuf))
 			{	non_fatal("mtype name %s too long", n->lft->sym->name);
 				break;
@@ -606,20 +680,17 @@ sr_buf(int v, int j)
 			sprintf(lbuf, "%s", n->lft->sym->name);
 			strcat(Buf, lbuf);
 			return;
-		}
+	}	}
 	sprintf(lbuf, "%d", v);
 	strcat(Buf, lbuf);
 }
 
 void
-sr_mesg(FILE *fd, int v, int j)
+sr_mesg(FILE *fd, int v, int j, const char *s)
 {	Buf[0] ='\0';
-	sr_buf(v, j);
-#if 1
+
+	sr_buf(v, j, s);
 	fprintf(fd, Buf, (char *) 0); /* prevent compiler warning */
-#else
-	fprintf(fd, Buf);
-#endif
 }
 
 void
@@ -634,27 +705,31 @@ doq(Symbol *s, int n, RunList *r)
 	{	if (xspin > 0
 		&& (verbose&4)
 		&& q->setat < depth)
-			continue;
-		if (q->nslots == 0)
-			continue; /* rv q always empty */
-#if 0
-		if (q->qlen == 0)	/* new 7/10 -- dont show if queue is empty */
 		{	continue;
 		}
-#endif
+		if (q->nslots == 0)
+		{	continue;	/* rv q always empty */
+		}
+
 		printf("\t\tqueue %d (", q->qid);
 		if (r)
-		printf("%s(%d):", r->n->name, r->pid - Have_claim);
+		{	printf("%s(%d):", r->n->name, r->pid - Have_claim);
+		}
+
 		if (s->nel > 1 || s->isarray)
-		  printf("%s[%d]): ", s->name, n);
-		else
-		  printf("%s): ", s->name);
+		{	printf("%s[%d]): ", s->name, n);
+		} else
+		{	printf("%s): ", s->name);
+		}
+
 		for (k = 0; k < q->qlen; k++)
 		{	printf("[");
 			for (j = 0; j < q->nflds; j++)
 			{	if (j > 0) printf(",");
-				sr_mesg(stdout, q->contents[k*q->nflds+j],
-					q->fld_width[j] == MTYPE);
+				sr_mesg(stdout,
+					q->contents[k*q->nflds+j],
+					q->fld_width[j] == MTYPE,
+					q->mtp[j]);
 			}
 			printf("]");
 		}
@@ -664,7 +739,7 @@ doq(Symbol *s, int n, RunList *r)
 }
 
 void
-nochan_manip(Lextok *p, Lextok *n, int d)
+nochan_manip(Lextok *p, Lextok *n, int d)	/* p=lhs n=rhs */
 {	int e = 1;
 
 	if (d == 0 && p->sym && p->sym->type == CHAN)
@@ -679,8 +754,30 @@ nochan_manip(Lextok *p, Lextok *n, int d)
 		}	
 	}
 
+	if (!d && n && n->ismtyp)	/* rhs is an mtype value (a constant) */
+	{	char *lhs = "_unnamed_", *rhs = "_unnamed_";
+
+		if (p->sym)
+		{	lhs = p->sym->mtype_name?p->sym->mtype_name->name:"_unnamed_";
+		}
+		if (n->sym)
+		{	rhs = which_mtype(n->sym->name); /* only for constants */
+		}
+
+		if (p->sym && !p->sym->mtype_name && n->sym)
+		{	p->sym->mtype_name = (Symbol *) emalloc(sizeof(Symbol));
+			p->sym->mtype_name->name = rhs;
+		} else if (strcmp(lhs, rhs) != 0)
+		{	fprintf(stderr, "spin: %s:%d, Error: '%s' is type '%s' but '%s' is type '%s'\n",
+				p->fn->name, p->ln,
+				p->sym?p->sym->name:"?", lhs,
+				n->sym?n->sym->name:"?", rhs);
+			non_fatal("type error", (char *) 0);
+	}	}
+
 	/* ok on the rhs of an assignment: */
-	if (!n || n->ntyp == LEN || n->ntyp == RUN
+	if (!n
+	||  n->ntyp == LEN   || n->ntyp == RUN
 	||  n->ntyp == FULL  || n->ntyp == NFULL
 	||  n->ntyp == EMPTY || n->ntyp == NEMPTY
 	||  n->ntyp == 'R')
