@@ -140,9 +140,12 @@ here:
 
 		i = blog(mst);
 		fprintf(th, "\n");
+
 		fprintf(th, "#ifndef NOCLAIM\n");
+		fprintf(th, " #ifndef NP\n");
 		fprintf(th, "	#undef VERI\n");
 		fprintf(th, "	#define VERI	%d\n", nrRdy+1);
+		fprintf(th, " #endif\n");
 		fprintf(th, "	#define Pclaim	P%d\n\n", nrRdy+1);
 		fprintf(th, "typedef struct P%d {\n", nrRdy+1);
 		fprintf(th, "	unsigned _pid : 8; /* always zero */\n");
@@ -643,11 +646,18 @@ c_chandump(FILE *fd)
 		fprintf(fd, "{	printf(\" [\");\n\t\t");
 		for (i = 0; i < q->nflds; i++)
 		{	if (q->fld_width[i] == MTYPE)
-			{	fprintf(fd, "\tprintm(%scontents[slot].fld%d);\n\t\t",
-				buf, i);
+			{	fprintf(fd, "\tprintm(%scontents[slot].fld%d",
+					buf, i);
+				if (q->mtp[i])
+				{	fprintf(fd, ", \"%s\"", q->mtp[i]);
+				} else
+				{	fprintf(fd, ", 0");
+				}
 			} else
-			fprintf(fd, "\tprintf(\"%%d,\", %scontents[slot].fld%d);\n\t\t",
-				buf, i);
+			{	fprintf(fd, "\tprintf(\"%%d,\", %scontents[slot].fld%d",
+					buf, i);
+			}
+			fprintf(fd, ");\n\t\t");
 		}
 		fprintf(fd, "	printf(\"],\");\n\t\t");
 		fprintf(fd, "}\n\t\t");
@@ -766,12 +776,13 @@ c_splurge(FILE *fd, ProcList *p)
 
 void
 c_wrapper(FILE *fd)	/* allow pan.c to print out global sv entries */
-{	Ordered *walk;
+{	Ordered  *walk;
 	ProcList *p;
-	Symbol *sp;
-	Lextok *n;
-	extern Lextok *Mtype;
-	int j;
+	Symbol   *sp;
+	Mtypes_t *lst;
+	Lextok   *n;
+	int	  j;
+	extern Mtypes_t *Mtypes;
 
 	fprintf(fd, "void\nc_globals(void)\n{\t/* int i; */\n");
 	fprintf(fd, "	printf(\"global vars:\\n\");\n");
@@ -798,13 +809,17 @@ c_wrapper(FILE *fd)	/* allow pan.c to print out global sv entries */
 	}
 	fprintf(fd, "	}\n}\n");
 
-	fprintf(fd, "void\nprintm(int x)\n{\n");
-	fprintf(fd, "	switch (x) {\n");
-        for (n = Mtype, j = 1; n && j; n = n->rgt, j++)
-                fprintf(fd, "\tcase %d: Printf(\"%s\"); break;\n",
-			j, n->lft->sym->name);
-	fprintf(fd, "	default: Printf(\"%%d\", x);\n");
-	fprintf(fd, "	}\n");
+	fprintf(fd, "void\nprintm(int x, char *s)\n{\n");
+	fprintf(fd, "	if (!s) { s = \"_unnamed_\"; }\n");
+	for (lst = Mtypes; lst; lst = lst->nxt)
+	{	fprintf(fd, "	if (strcmp(s, \"%s\") == 0)\n", lst->nm);
+		fprintf(fd, "	switch (x) {\n");
+	        for (n = lst->mt, j = 1; n && j; n = n->rgt, j++)
+	                fprintf(fd, "\tcase %d: Printf(\"%s\"); return;\n",
+				j, n->lft->sym->name);
+		fprintf(fd, "	default: Printf(\"%%d\", x); return;\n");
+		fprintf(fd, "	}\n");
+	}
 	fprintf(fd, "}\n");
 }
 
@@ -976,9 +991,9 @@ put_ptype(char *s, int i, int m0, int m1, enum btypes b)
 {	int k;
 
 	if (b == I_PROC)
-	{	fprintf(th, "#define Pinit	((P%d *)this)\n", i);
+	{	fprintf(th, "#define Pinit	((P%d *)_this)\n", i);
 	} else if (b == P_PROC || b == A_PROC)
-	{	fprintf(th, "#define P%s	((P%d *)this)\n", s, i);
+	{	fprintf(th, "#define P%s	((P%d *)_this)\n", s, i);
 	}
 
 	fprintf(th, "typedef struct P%d { /* %s */\n", i, s);
@@ -1086,8 +1101,8 @@ multi_init(void)
 			  fprintf(tc, "\t\tsrc_claim = src_ln%d;\n", p->tn);
 			  fprintf(tc, "#ifndef BFS\n");
 			  fprintf(tc, "\t\tif (whichclaim == -1 && claimname == NULL)\n");
-			  fprintf(tc, "\t\t\tprintf(\"0: Claim %s (%d), from state %d\\n\");\n",
-				p->n->name, p->tn, ini);
+			  fprintf(tc, "\t\t\tprintf(\"pan: ltl fomula %s\\n\");\n",
+				p->n->name);
 			  fprintf(tc, "#endif\n");
 			}
 	}	}
@@ -1502,7 +1517,13 @@ genaddqueue(void)
 		fprintf(tc, "\t\t%sQlen = %sQlen + 1;\n", buf0, buf0);
 		sprintf(buf0, "((Q%d *)z)->contents[j].fld", q->qid);
 		for (j = 0; j < q->nflds; j++)
-			fprintf(tc, "\t\t%s%d = fld%d;\n", buf0, j, j);
+		{	fprintf(tc, "\t\t%s%d = fld%d;", buf0, j, j);
+			if (q->fld_width[j] == MTYPE)
+			{	fprintf(tc, "\t/* mtype %s */",
+					q->mtp[j]?q->mtp[j]:"_unnamed_");
+			}
+			fprintf(tc, "\n");
+		}
 		fprintf(tc, "\t\tif (args_given != %d)\n", q->nflds);
 		fprintf(tc, "\t\t{	if (args_given > %d)\n", q->nflds);
 		fprintf(tc, "\t\t		uerror(\"too many parameters in send stmnt\");\n");
