@@ -62,6 +62,7 @@ int	claimnr = -1;	/* claim process, if any */
 int	eventmapnr = -1; /* event trace, if any */
 int	Pid;		/* proc currently processed */
 int	multi_oval;	/* set in merges, used also in pangen4.c */
+int	in_settr;	/* avoid quotes inside quotes */
 
 #define MAXMERGE	256	/* max nr of bups per merge sequence */
 
@@ -81,6 +82,7 @@ static short	_isok=0;	/* checks usage of predefined variable _ */
 static short	evalindex=0;	/* evaluate index of var names */
 
 int	has_global(Lextok *);
+extern void	check_mtypes(Lextok *, Lextok *);
 static int	getweight(Lextok *);
 static int	scan_seq(Sequence *);
 static void	genconditionals(void);
@@ -179,7 +181,7 @@ tt_predef_np(void)
 {
 	fprintf(tt, "\t/* np_ demon: */\n");
 	fprintf(tt, "\ttrans[_NP_] = ");
-	fprintf(tt, "(Trans **) emalloc(2*sizeof(Trans *));\n");
+	fprintf(tt, "(Trans **) emalloc(3*sizeof(Trans *));\n");
 	fprintf(tt, "\tT = trans[_NP_][0] = ");
 	fprintf(tt, "settr(9997,0,1,_T5,0,\"(np_)\", 1,2,0);\n");
 	fprintf(tt, "\t    T->nxt	  = ");
@@ -427,6 +429,10 @@ doless:
 	fprintf(th, "	#define HAS_NP	2\n");
 	fprintf(th, "	#define VERI	%d	/* np_ */\n",	nrRdy);
 	fprintf(th, "#endif\n");
+
+	fprintf(th, "#if defined(NOCLAIM) && defined(NP)\n");
+	fprintf(th, "	#undef NOCLAIM\n");
+	fprintf(th, "#endif\n");
 	if (claimproc)
 	{	claimnr = fproc(claimproc);	/* the default claim */
 		fprintf(th, "#ifndef NOCLAIM\n");
@@ -773,7 +779,7 @@ dolen(Symbol *s, char *pre, int pid, int ai, int qln)
 	fprintf(tc, "%s(", pre);
 	if (!(s->hidden&1))
 	{	if (s->context)
-			fprintf(tc, "(int) ( ((P%d *)this)->", pid);
+			fprintf(tc, "(int) ( ((P%d *)_this)->", pid);
 		else
 			fprintf(tc, "(int) ( now.");
 	}
@@ -933,7 +939,7 @@ genconditionals(void)
 			strcpy(pregat, "");
 			if (!(s->hidden&1))
 			{	if (s->context)
-					sprintf(pregat, "((P%d *)this)->",j);
+					sprintf(pregat, "((P%d *)_this)->",j);
 				else
 					sprintf(pregat, "now.");
 			}
@@ -1231,7 +1237,9 @@ put_sub(Element *e, int Tt0, int Tt1)
 			Pid, e->seqno);
 		fprintf(tt, "settr(%d,%d,%d,%d,%d,\"",
 			e->Seqno, D_ATOM|inherit, a, uniq-1, uniq-1);
+in_settr++;
 		comment(tt, e->n, e->seqno);
+in_settr--;
 		fprintf(tt, "\", %d, ", (s->frst->status&I_GLOB)?1:0);
 		fprintf(tt, "%d, %d);\n", TPE[0], TPE[1]);
 		put_escp(e);
@@ -1240,7 +1248,9 @@ put_sub(Element *e, int Tt0, int Tt1)
 		fprintf(tt, "\tT = trans[ %d][%d] = ", Pid, e->seqno);
 		fprintf(tt, "settr(%d,%d,0,0,0,\"",
 			e->Seqno, (e->n->ntyp == ATOMIC)?ATOM:0);
+in_settr++;
 		comment(tt, e->n, e->seqno);
+in_settr--;
 		if ((e->status&CHECK2)
 		||  (g->status&CHECK2))
 			s->frst->status |= I_GLOB;
@@ -1250,7 +1260,9 @@ put_sub(Element *e, int Tt0, int Tt1)
 		fprintf(tt, "\tT->nxt\t= ");
 		fprintf(tt, "settr(%d,%d,%d,0,0,\"",
 			e->Seqno, (e->n->ntyp == ATOMIC)?ATOM:0, a);
+in_settr++;
 		comment(tt, e->n, e->seqno);
+in_settr--;
 		fprintf(tt, "\", %d, ", (s->frst->status&I_GLOB)?1:0);
 		if (e->n->ntyp == NON_ATOMIC)
 		{	fprintf(tt, "%d, %d);", Tt0, Tt1);
@@ -1675,7 +1687,9 @@ case_cache(Element *e, int a)
 		&&  e->n->ntyp != GOTO)
 		{	fprintf(tt, "\ttrans[%d][%d]\t= ", Pid, e->seqno);
 			fprintf(tt, "settr(0,0,0,0,0,\"");
+in_settr++;
 			comment(tt, e->n, e->seqno);
+in_settr--;
 			fprintf(tt, "\",0,0,0);\n");
 		} else
 		{	fprintf(tt, "\ttrans[%d][%d]\t= ", Pid, e->seqno);
@@ -1916,7 +1930,9 @@ non_generic:
 	}
 	/* tailend of settr(...); */
 	Global_ref = (e->status&I_GLOB)?1:has_global(e->n);
+in_settr++;
 	comment(tt, e->n, e->seqno);
+in_settr--;
 	fprintf(tt, "\", %d, ", Global_ref);
 	if (Tt0 != 2)
 	{	fprintf(tt, "%d, %d);", Tt0, Tt1);
@@ -2000,7 +2016,9 @@ put_seq(Sequence *s, int Tt0, int Tt1)
 				Pid, e->seqno);
 			fprintf(tt, "settr(%d,%d,0,0,0,\"",
 				e->Seqno, e->status&ATOM);
+in_settr++;
 			comment(tt, e->n, e->seqno);
+in_settr--;
 			if (e->status&CHECK2)
 				e->status |= I_GLOB;
 			fprintf(tt, "\", %d, %d, %d);",
@@ -2031,7 +2049,9 @@ put_seq(Sequence *s, int Tt0, int Tt1)
 					fprintf(tt, "\t    T->nxt\t= ");
 				fprintf(tt, "settr(%d,%d,%d,0,0,\"",
 					e->Seqno, e->status&ATOM, a);
+in_settr++;
 				comment(tt, e->n, e->seqno);
+in_settr--;
 				if (g->status&CHECK2)
 					h->this->frst->status |= I_GLOB;
 				fprintf(tt, "\", %d, %d, %d);",
@@ -2415,6 +2435,7 @@ putstmnt(FILE *fd, Lextok *now, int m)
 		for ( ; i < Npars; i++)
 			fprintf(fd, ", 0");
 		fprintf(fd, ")");
+		check_mtypes(now, now->lft);
 #if 0
 		/* process now->sym->name has run priority now->val */
 		if (now->val > 0 && now->val < 256 && !old_priority_rules)
@@ -2605,19 +2626,23 @@ putstmnt(FILE *fd, Lextok *now, int m)
 			fatal("too many pars in send", "");
 		}
 		for (j = i; i < Mpars; i++)
-			fprintf(fd, ", 0");
+		{	fprintf(fd, ", 0");
+		}
 		fprintf(fd, ", %d)", j);
 		if (u_sync)
 		{	fprintf(fd, ";\n\t\t");
 			if (u_async)
-			  putname(fd, "if (q_zero(", now->lft, m, ")) ");
+			{	putname(fd, "if (q_zero(", now->lft, m, ")) ");
+			}
 			putname(fd, "{ boq = ", now->lft, m, "");
 			if (GenCode)
-			  fprintf(fd, "; Uerror(\"rv-attempt in d_step\")");
+			{	fprintf(fd, "; Uerror(\"rv-attempt in d_step\")");
+			}
 			fprintf(fd, "; }");
 		}
 		if (m_loss)
-			fprintf(fd, ";\n\t\t}\n\t\t"); /* end of m_loss else */
+		{	fprintf(fd, ";\n\t\t}\n\t\t"); /* end of m_loss else */
+		}
 		break;
 
 	case 'r':
@@ -2927,7 +2952,7 @@ putstmnt(FILE *fd, Lextok *now, int m)
 			fprintf(fd, "\t\t\t		now._cnt[now._a_t&1] = 1;\n");
 			fprintf(fd, "#endif\n");
 			fprintf(fd, "#ifdef DEBUG\n");
-			fprintf(fd, "\t\t\tprintf(\"%%3d: proc %%d fairness \", depth, II);\n");
+			fprintf(fd, "\t\t\tprintf(\"%%3ld: proc %%d fairness \", depth, II);\n");
 			fprintf(fd, "\t\t\tprintf(\"Rule 2: --cnt to %%d (%%d)\\n\",\n");
 			fprintf(fd, "\t\t\t	now._cnt[now._a_t&1], now._a_t);\n");
 			fprintf(fd, "#endif\n");
@@ -3085,14 +3110,29 @@ putstmnt(FILE *fd, Lextok *now, int m)
 		break;
 
 	case PRINTM:
-		if (has_enabled || has_priority)
-			fprintf(fd, "if (TstOnly) return 1; /* T5 */\n\t\t");
-		fprintf(fd, "printm(");
-		if (now->lft && now->lft->ismtyp)
-			fprintf(fd, "%d", now->lft->val);
-		else
-			putstmnt(fd, now->lft, m);
-		fprintf(fd, ")");
+		{ char *s = 0;
+		  if (now->lft->sym
+		  &&  now->lft->sym->mtype_name)
+		  {	s = now->lft->sym->mtype_name->name;
+		  }
+
+		  if (has_enabled || has_priority)
+		  {	fprintf(fd, "if (TstOnly) return 1; /* T5 */\n\t\t");
+		  }
+		  fprintf(fd, "/* YY */ printm(");
+		  if (now->lft
+		  &&  now->lft->ismtyp)
+		  {	fprintf(fd, "%d", now->lft->val);
+		  } else
+		  {	putstmnt(fd, now->lft, m);
+		  }
+		  if (s)
+		  {	fprintf(fd, ", \"%s\"", s);
+		  } else
+		  {	fprintf(fd, ", 0");
+		  }
+		  fprintf(fd, ")");
+		}
 		break;
 
 	case NAME:
@@ -3235,7 +3275,7 @@ putname(FILE *fd, char *pre, Lextok *n, int m, char *suff) /* varref */
 			||  strcmp(s->name, "_p") == 0
 			||  strcmp(s->name, "_pid") == 0
 			||  strcmp(s->name, "_priority") == 0)
-			{	fprintf(fd, "((P%d *)this)->", Pid);
+			{	fprintf(fd, "((P%d *)_this)->", Pid);
 			} else
 			{	int x = strcmp(s->name, "_");
 				if (!(s->hidden&1) && x != 0)
