@@ -134,7 +134,7 @@ static const char *Preamble[] = {
 	"/* #endif */",
 	"Trail	*trail, *trpt;",
 	"FILE	*efd;",
-	"uchar	*this;",
+	"uchar	*_this;",
 	"long	maxdepth=10000;",
 	"long	omaxdepth=10000;",
 	"#ifdef BCS",
