@@ -519,7 +519,8 @@ FSM_EDGE(int from, int to, Element *e)
 	}
 
 	if (t->step)
-		ana_stmnt(t, t->step->n, 0);
+	{	ana_stmnt(t, t->step->n, 0);
+	}
 }
 
 #define LVAL	1
@@ -698,9 +699,9 @@ ana_stmnt(FSM_trans *t, Lextok *now, int usage)
 		break;
 
 	default:
-		printf("spin: %s:%d, bad node type %d (ana_stmnt)\n",
-			now->fn->name, now->ln, now->ntyp);
-		fatal("aborting", (char *) 0);
+		if (0) printf("spin: %s:%d, bad node type %d usage %d (ana_stmnt)\n",
+			now->fn->name, now->ln, now->ntyp, usage);
+		fatal("aborting (ana_stmnt)", (char *) 0);
 	}
 }
 
