@@ -219,7 +219,6 @@ static const char *Code2c[] = { /* multi-core option - Spin 5.0 and later */
 	"int	store_proxy_pid;",
 	"short	remote_party;",
 	"int	proxy_pid_snd;		/* id of proxy if nonzero -- send half */",
-	"char	o_cmdline[512];		/* to pass options to children */",
 	"",
 	"int	iamin[CS_NR+NCORE];		/* non-shared */",
 	"",
@@ -1146,7 +1145,7 @@ static const char *Code2c[] = { /* multi-core option - Spin 5.0 and later */
 	"			}",
 	"			SEG_SIZE /= 2.;",
 	"			if (verbose)",
-	"			{	printf(\"pan: lowered segsize to %f\\n\", SEG_SIZE);",
+	"			{	printf(\"pan: lowered segsize to %%f\\n\", SEG_SIZE);",
 	"			}",
 	"			if (SEG_SIZE >= 1024.)",
 	"			{	goto shm_more;", /* always terminates */
