@@ -848,9 +848,9 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"	if (boq == -1 && !(trpt->tau&8) && bfs_nps == 0)",
 	"	for (II = now._nr_pr-1; II >= BASE; II -= 1)",
 	"	{",
-	"Pickup:	this = pptr(II);",
-	"		tt = (int) ((P0 *)this)->_p;",
-	"		ot = (uchar) ((P0 *)this)->_t;",
+	"Pickup:	_this = pptr(II);",
+	"		tt = (int) ((P0 *)_this)->_p;",
+	"		ot = (uchar) ((P0 *)_this)->_t;",
 	"		if (trans[ot][tt]->atom & 8)",
 	"		{	t = trans[ot][tt];",
 	"			if (t->qu[0] != 0)",
@@ -889,9 +889,9 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"		{	continue;	/* no rendezvous with same proc */",
 	"		}",
 	"#endif",
-	"		this = pptr(II);",
-	"		tt = (int) ((P0 *)this)->_p;",
-	"		ot = (uchar) ((P0 *)this)->_t;",
+	"		_this = pptr(II);",
+	"		tt = (int) ((P0 *)_this)->_p;",
+	"		ot = (uchar) ((P0 *)_this)->_t;",
 	"		ntrpt->pr = (uchar) II;",
 	"		ntrpt->st = tt;	",
 	"		trpt->o_pm &= ~1; /* no move yet */",
@@ -899,7 +899,7 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"		trpt->o_event = now._event;",
 	"#endif",
 	"#ifdef HAS_PRIORITY",
-	"		if (!highest_priority(((P0 *)this)->_pid, II, t))",
+	"		if (!highest_priority(((P0 *)_this)->_pid, II, t))",
 	"		{	continue;",
 	"		}",
 	"#else",
@@ -963,7 +963,7 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"	#endif",
 	"#endif",
 	"			if (t->st > 0)",
-	"			{	((P0 *)this)->_p = t->st;",
+	"			{	((P0 *)_this)->_p = t->st;",
 	"			}",
 	"			/* use the ostate ptr, with type *H_el, to temporarily store *BFS_Trail */",
 	"#ifdef BFS_NOTRAIL",
@@ -1022,7 +1022,7 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"			reached[ot][t->st] = 1;",
 	"			reached[ot][tt]    = 1;",
 	"",
-	"			((P0 *)this)->_p = tt;",
+	"			((P0 *)_this)->_p = tt;",
 	"			_n |= _m;",
 	"	}	}",
 	"#ifdef VERBOSE",
@@ -1055,8 +1055,8 @@ static const char *pan_par[] = {	/* generates pan.p */
 	"		{	BFS_Trail *x = (BFS_Trail *) trpt->ostate; /* pre-rv state */",
 	"			if (x && ((x->tau&8) || (x->tau&32))) /* break atomic or preselect */",
 	"			{	x->o_pm |= 8;		/* mark failure */",
-	"				this = pptr(otrpt->pr);",
-	"				((P0 *) this)->_p = otrpt->st;	/* reset state */",
+	"				_this = pptr(otrpt->pr);",
+	"				((P0 *) _this)->_p = otrpt->st;	/* reset state */",
 	"				unsend(boq);		/* retract rv offer */",
 	"				boq = -1;",
 	"#ifdef VERBOSE",
