@@ -25,7 +25,7 @@ static int	eval_sync(Element *);
 static int	pc_enabled(Lextok *n);
 static int	get_priority(Lextok *n);
 static void	set_priority(Lextok *n, Lextok *m);
-extern void	sr_buf(int, int);
+extern void	sr_buf(int, int, const char *);
 
 void
 Srand(unsigned int s)
@@ -465,16 +465,24 @@ eval(Lextok *now)
 int
 printm(FILE *fd, Lextok *n)
 {	extern char Buf[];
+	char *s = 0;
 	int j;
 
 	Buf[0] = '\0';
 	if (!no_print)
-	if (!s_trail || depth >= jumpsteps) {
+	if (!s_trail || depth >= jumpsteps)
+	{
+		if (n->lft->sym
+		&&  n->lft->sym->mtype_name)
+		{	s = n->lft->sym->mtype_name->name;
+		}
+
 		if (n->lft->ismtyp)
-			j = n->lft->val;
-		else
-			j = eval(n->lft);
-		sr_buf(j, 1);
+		{	j = n->lft->val;
+		} else	/* constant */
+		{	j = eval(n->lft);
+		}
+		sr_buf(j, 1, s);
 		dotag(fd, Buf);
 	}
 	return 1;
@@ -483,7 +491,7 @@ printm(FILE *fd, Lextok *n)
 int
 interprint(FILE *fd, Lextok *n)
 {	Lextok *tmp = n->lft;
-	char c, *s = n->sym->name;
+	char c, *s = n->sym->name, *t = 0;
 	int i, j; char lbuf[512]; /* matches value in sr_buf() */
 	extern char Buf[];	/* global, size 4096 */
 	char tBuf[4096];	/* match size of global Buf[] */
@@ -511,6 +519,14 @@ interprint(FILE *fd, Lextok *n)
 				break;
 			 }
 			 j = eval(tmp->lft);
+
+			 if (c == 'e'
+			 &&  tmp->lft
+			 &&  tmp->lft->sym
+			 &&  tmp->lft->sym->mtype_name)
+			 {	t = tmp->lft->sym->mtype_name->name;
+			 }
+
 			 tmp = tmp->rgt;
 			 switch(c) {
 			 case 'c': sprintf(lbuf, "%c", j); break;
@@ -518,7 +534,9 @@ interprint(FILE *fd, Lextok *n)
 
 			 case 'e': strcpy(tBuf, Buf);	/* event name */
 				   Buf[0] = '\0';
-				   sr_buf(j, 1);
+
+				   sr_buf(j, 1, t);
+
 				   strcpy(lbuf, Buf);
 				   strcpy(Buf, tBuf);
 				   break;
