@@ -126,10 +126,16 @@ Getchar(void)
 static void
 Ungetch(int c)
 {
+	if (PushedBack > 0 && PushBack > 0)
+	{	PushBack--;
+		return;
+	}
+
 	if (Inlining<0)
-		ungetc(c,yyin);
-	else
-		uninline();
+	{	ungetc(c,yyin);
+	} else
+	{	uninline();
+	}
 	if (0)
 	{	printf("\n<bs{%d}bs>\n", c);
 	}
@@ -172,6 +178,7 @@ getword(int first, int (*tst)(int))
 			yytext[i++] = (char) c;	/* no tst */
 	}	}
 	yytext[i] = '\0';
+
 	Ungetch(c);
 }
 
@@ -465,7 +472,10 @@ jump_etc(C_Added *r)
 		{	fatal("c_state format (%s)", op);
 	}	}
 
-	if (strchr(p, '[') && !strchr(p, '{'))
+	if (strchr(p, '[')
+	&& (!r->ival
+	||  !r->ival->name
+	||  !strchr(r->ival->name, '{')))	// was !strchr(p, '{'))
 	{	non_fatal("array initialization error, c_state (%s)", p);
 		p = (char *) 0;
 	}
@@ -526,7 +536,6 @@ c_add_locinit(FILE *fd, int tpnr, char *pnm)
 		{	for (q = r->ival->name; *q; q++)
 				if (*q == '\"')
 					*q = ' ';
-			
 			p = jump_etc(r);	/* e.g., "int **q" */
 
 			q = r->t->name + strlen(" Local");
@@ -544,12 +553,12 @@ c_add_locinit(FILE *fd, int tpnr, char *pnm)
 				continue;
 
 			if (frst)
-			{	fprintf(fd, "\tuchar *this = pptr(h);\n");
+			{	fprintf(fd, "\tuchar *_this = pptr(h);\n");
 				frst = 0;
 			}
 
 			if (p)
-			{	fprintf(fd, "\t\t((P%d *)this)->%s = %s;\n",
+			{	fprintf(fd, "\t\t((P%d *)_this)->%s = %s;\n",
 					tpnr, p, r->ival->name);
 			}
 		}
@@ -686,14 +695,16 @@ c_add_loc(FILE *fd, char *s)	/* state vector entries for proctype s */
 	strcpy(buf, s);
 	strcat(buf, " ");
 	for (r = c_added; r; r = r->nxt)	/* pickup local decls */
-		if (strncmp(r->t->name, " Local", strlen(" Local")) == 0)
+	{	if (strncmp(r->t->name, " Local", strlen(" Local")) == 0)
 		{	p = r->t->name + strlen(" Local");
 fprintf(fd, "/* XXX p=<%s>, s=<%s>, buf=<%s> r->s->name=<%s>XXX */\n", p, s, buf, r->s->name);
 			while (*p == ' ' || *p == '\t')
-				p++;
-			if (strcmp(p, buf) == 0)
-				fprintf(fd, "	%s;\n", r->s->name);
-		}
+			{	p++;
+			}
+			if (strcmp(p, buf) == 0
+			||  (strncmp(p, "init", 4) == 0 && strncmp(buf, ":init:", 6) == 0))
+			{	fprintf(fd, "	%s;\n", r->s->name);
+	}	}	}
 }
 void
 c_add_def(FILE *fd)	/* 3 - called in plunk_c_fcts() */
@@ -1009,6 +1020,7 @@ void
 no_side_effects(char *s)
 {	IType *tmp;
 	char *t;
+	char *z;
 
 	/* could still defeat this check via hidden
 	 * side effects in function calls,
@@ -1017,6 +1029,17 @@ no_side_effects(char *s)
 
 	tmp = find_inline(s);
 	t = (char *) tmp->cn;
+	while (t && *t == ' ')
+	{	t++;
+	}
+
+	z = strchr(t, '(');
+	if (z
+	&&  z > t
+	&&  isalnum((int) *(z-1))
+	&&  strncmp(t, "spin_mutex_free(", strlen("spin_mutex_free(")) != 0)
+	{	goto bad;	/* fct call */
+	}
 
 	if (side_scan(t, ";")
 	||  side_scan(t, "++")
@@ -1450,6 +1473,7 @@ lex(void)
 {	int c;
 again:
 	c = Getchar();
+more:
 	yytext[0] = (char) c;
 	yytext[1] = '\0';
 	switch (c) {
@@ -1473,7 +1497,20 @@ again:
 		&&  in_seq
 		&&  par_cnt == 0
 		&&  follows_token(last_token))
-		{	if (0)
+		{	if (last_token == '}')
+			{	do {	c = Getchar();
+					if (c == '\n')
+					{	lineno++;
+					}
+				} while (c == ' ' || c == '\t' ||
+					 c == '\f' || c == '\n' ||
+					 c == '\r');
+				Ungetch(c);
+				if (0) printf("%d: saw %d\n", lineno, c);
+				if (c == 'u') /* first letter of UNLESS */
+				{	goto again;
+			}	}
+			if (0)
 			{	printf("insert ; line %d, last_token %d in_seq %d\n",
 				 lineno-1, last_token, in_seq);
 			}
@@ -1543,6 +1580,16 @@ again:
 		if (!in_comment)
 		{	c = check_name(yytext);
 
+// replace timeout with (timeout)
+			if (c == TIMEOUT
+			&&  Inlining < 0
+			&&  last_token != '(')
+			{	push_back("timeout)");
+				last_token = '(';
+				return '(';
+			}
+// end
+
 #ifdef EXPAND_SELECT
 			if (c == SELECT && Inlining < 0)
 			{	char name[64], from[32], upto[32];
@@ -1577,13 +1624,13 @@ again:
 					push_back("fi ");
 				} else
 				{	char buf[128];
-					sprintf(buf, "%s = %d; do :: ",
+					sprintf(buf, "%s = %d; do ",
 						name, a);
 					push_back(buf);
-					sprintf(buf, "(%s < %d) -> %s++ ",
+					sprintf(buf, ":: (%s < %d) -> %s++ ",
 						name, b, name);
 					push_back(buf);
-					push_back(":: break od ");
+					push_back(":: break od; ");
 				}
 				goto again;
 			}
@@ -1767,6 +1814,9 @@ check_name(char *s)
 
 	if ((yylval->val = ismtype(s)) != 0)
 	{	yylval->ismtyp = 1;
+		yylval->sym = (Symbol *) emalloc(sizeof(Symbol));
+		yylval->sym->name = (char *) emalloc(strlen(s)+1);
+		strcpy(yylval->sym->name, s);
 		return CONST;
 	}
 
